@@ -23,7 +23,7 @@ __copyright__ = '(C) 2013, Victor Olaya'
 
 # This will get replaced with a git SHA1 when you do a git archive
 
-__revision__ = '26842169e9367434a27a70ab0df0bd4ba6ad1d7a'
+__revision__ = '564579199220ae06abd248f0dc80b7ff57b80128'
 
 from qgis.PyQt.QtCore import Qt, QCoreApplication
 from qgis.PyQt.QtWidgets import QProgressBar
@@ -45,8 +45,8 @@ class MessageBarProgress(QgsProcessingFeedback):
         self.progress.setMaximum(100)
         self.progress.setAlignment(Qt.AlignLeft | Qt.AlignVCenter)
         self.progressMessageBar.layout().addWidget(self.progress)
-        iface.messageBar().pushWidget(self.progressMessageBar,
-                                      Qgis.Info)
+        self.message_bar_item = iface.messageBar().pushWidget(self.progressMessageBar,
+                                                              Qgis.Info)
 
     def reportError(self, msg, fatalError=False):
         self.msg.append(msg)
@@ -57,7 +57,7 @@ class MessageBarProgress(QgsProcessingFeedback):
             dlg.setTitle(QCoreApplication.translate('MessageBarProgress', 'Problem executing algorithm'))
             dlg.setMessage("<br>".join(self.msg))
             dlg.exec_()
-        iface.messageBar().clearWidgets()
+        iface.messageBar().popWidget(self.message_bar_item)
 
     def tr(self, string, context=''):
         if context == '':
